@@ -49,11 +49,12 @@
 //!
 
 use std::collections::HashMap;
-use std::str::FromStr;
 
 use blanket::blanket;
 
 use crate::ast::*;
+use crate::parser::Cache;
+use crate::parser::QuickFind;
 
 // ---------------------------------------------------------------------------
 
@@ -64,6 +65,7 @@ use crate::ast::*;
 #[blanket(default = "visit", derive(Mut, Box))]
 pub trait Visit<'ast> {
     fn visit_class_ident(&mut self, id: &'ast ClassIdent);
+    fn visit_creation_date(&mut self, creation_date: &'ast CreationDate);
     fn visit_definition(&mut self, id: &'ast Definition);
     fn visit_doc(&mut self, doc: &'ast OboDoc);
     fn visit_entity_frame(&mut self, frame: &'ast EntityFrame);
@@ -75,7 +77,9 @@ pub trait Visit<'ast> {
     fn visit_instance_clause(&mut self, clause: &'ast InstanceClause);
     fn visit_instance_frame(&mut self, frame: &'ast InstanceFrame);
     fn visit_instance_ident(&mut self, id: &'ast InstanceIdent);
-    fn visit_iso_date(&mut self, date: &'ast IsoDateTime);
+    fn visit_iso_date(&mut self, date: &'ast IsoDate);
+    fn visit_iso_datetime(&mut self, datetime: &'ast IsoDateTime);
+    fn visit_iso_time(&mut self, time: &'ast IsoTime);
     fn visit_literal_property_value(&mut self, id: &'ast LiteralPropertyValue);
     fn visit_naive_date(&mut self, date: &'ast NaiveDateTime);
     fn visit_namespace_ident(&mut self, id: &'ast NamespaceIdent);
@@ -108,6 +112,17 @@ pub mod visit {
         visitor.visit_ident(id.as_ref())
     }
 
+    pub fn visit_creation_date<'ast, V: Visit<'ast> + ?Sized>(
+        visitor: &mut V,
+        creation_date: &'ast CreationDate,
+    ) {
+        use self::CreationDate::*;
+        match creation_date {
+            Date(ref d) => visitor.visit_iso_date(d.as_ref()),
+            DateTime(ref d) => visitor.visit_iso_datetime(d.as_ref()),
+        }
+    }
+
     pub fn visit_definition<'ast, V: Visit<'ast> + ?Sized>(visitor: &mut V, def: &'ast Definition) {
         visitor.visit_quoted_string(def.text());
         visitor.visit_xref_list(def.xrefs());
@@ -247,7 +262,7 @@ pub mod visit {
                 visitor.visit_ident(id);
             }
             CreatedBy(s) => visitor.visit_unquoted_string(s),
-            CreationDate(dt) => visitor.visit_iso_date(dt),
+            CreationDate(dt) => visitor.visit_creation_date(dt),
             IsObsolete(_) => (),
             ReplacedBy(id) => visitor.visit_instance_ident(id),
             Consider(id) => visitor.visit_ident(id),
@@ -272,9 +287,20 @@ pub mod visit {
     }
 
     #[allow(unused_variables)]
-    pub fn visit_iso_date<'ast, V: Visit<'ast> + ?Sized>(visitor: &mut V, date: &'ast IsoDateTime) {
+    pub fn visit_iso_date<'ast, V: Visit<'ast> + ?Sized>(visitor: &mut V, datetime: &'ast IsoDate) {
     }
 
+    pub fn visit_iso_datetime<'ast, V: Visit<'ast> + ?Sized>(
+        visitor: &mut V,
+        datetime: &'ast IsoDateTime,
+    ) {
+        visitor.visit_iso_date(datetime.as_ref());
+        visitor.visit_iso_time(datetime.as_ref());
+    }
+
+    #[allow(unused_variables)]
+    pub fn visit_iso_time<'ast, V: Visit<'ast> + ?Sized>(visitor: &mut V, time: &'ast IsoTime) {}
+
     pub fn visit_literal_property_value<'ast, V: Visit<'ast> + ?Sized>(
         visitor: &mut V,
         pv: &'ast LiteralPropertyValue,
@@ -348,7 +374,7 @@ pub mod visit {
     pub fn visit_synonym<'ast, V: Visit<'ast> + ?Sized>(visitor: &mut V, syn: &'ast Synonym) {
         visitor.visit_quoted_string(syn.description());
         visitor.visit_synonym_scope(syn.scope());
-        if let Some(ref id) = syn.ty() {
+        if let Some(id) = syn.ty() {
             visitor.visit_synonymtype_ident(id);
         }
         visitor.visit_xref_list(syn.xrefs())
@@ -403,7 +429,7 @@ pub mod visit {
             ReplacedBy(id) => visitor.visit_class_ident(id),
             Consider(id) => visitor.visit_class_ident(id),
             CreatedBy(s) => visitor.visit_unquoted_string(s),
-            CreationDate(dt) => visitor.visit_iso_date(dt),
+            CreationDate(dt) => visitor.visit_creation_date(dt),
         }
     }
 
@@ -462,7 +488,7 @@ pub mod visit {
             ReplacedBy(id) => visitor.visit_relation_ident(id),
             Consider(id) => visitor.visit_ident(id),
             CreatedBy(s) => visitor.visit_unquoted_string(s),
-            CreationDate(dt) => visitor.visit_iso_date(dt),
+            CreationDate(dt) => visitor.visit_creation_date(dt),
             ExpandAssertionTo(s, xrefs) | ExpandExpressionTo(s, xrefs) => {
                 visitor.visit_quoted_string(s);
                 visitor.visit_xref_list(xrefs);
@@ -501,7 +527,7 @@ pub mod visit {
 
     pub fn visit_xref<'ast, V: Visit<'ast> + ?Sized>(visitor: &mut V, xref: &'ast Xref) {
         visitor.visit_ident(xref.id());
-        if let Some(ref d) = xref.description() {
+        if let Some(d) = xref.description() {
             visitor.visit_quoted_string(d);
         }
     }
@@ -522,6 +548,7 @@ pub mod visit {
 #[blanket(default = "visit_mut", derive(Mut, Box))]
 pub trait VisitMut {
     fn visit_class_ident(&mut self, id: &mut ClassIdent);
+    fn visit_creation_date(&mut self, creation_date: &mut CreationDate);
     fn visit_definition(&mut self, id: &mut Definition);
     fn visit_doc(&mut self, doc: &mut OboDoc);
     fn visit_entity_frame(&mut self, frame: &mut EntityFrame);
@@ -533,7 +560,9 @@ pub trait VisitMut {
     fn visit_instance_clause(&mut self, clause: &mut InstanceClause);
     fn visit_instance_frame(&mut self, frame: &mut InstanceFrame);
     fn visit_instance_ident(&mut self, id: &mut InstanceIdent);
-    fn visit_iso_date(&mut self, date: &mut IsoDateTime);
+    fn visit_iso_date(&mut self, date: &mut IsoDate);
+    fn visit_iso_datetime(&mut self, datetime: &mut IsoDateTime);
+    fn visit_iso_time(&mut self, time: &mut IsoTime);
     fn visit_literal_property_value(&mut self, id: &mut LiteralPropertyValue);
     fn visit_naive_date(&mut self, date: &mut NaiveDateTime);
     fn visit_namespace_ident(&mut self, id: &mut NamespaceIdent);
@@ -566,6 +595,17 @@ pub mod visit_mut {
         visitor.visit_ident(id.as_mut())
     }
 
+    pub fn visit_creation_date<V: VisitMut + ?Sized>(
+        visitor: &mut V,
+        creation_date: &mut CreationDate,
+    ) {
+        use self::CreationDate::*;
+        match creation_date {
+            Date(d) => visitor.visit_iso_date(d.as_mut()),
+            DateTime(dt) => visitor.visit_iso_datetime(dt.as_mut()),
+        }
+    }
+
     pub fn visit_definition<V: VisitMut + ?Sized>(visitor: &mut V, def: &mut Definition) {
         visitor.visit_quoted_string(def.text_mut());
         visitor.visit_xref_list(def.xrefs_mut());
@@ -692,7 +732,7 @@ pub mod visit_mut {
                 visitor.visit_ident(id);
             }
             CreatedBy(s) => visitor.visit_unquoted_string(s),
-            CreationDate(dt) => visitor.visit_iso_date(dt),
+            CreationDate(dt) => visitor.visit_creation_date(dt),
             IsObsolete(_) => (),
             ReplacedBy(id) => visitor.visit_instance_ident(id),
             Consider(id) => visitor.visit_ident(id),
@@ -710,8 +750,16 @@ pub mod visit_mut {
         visitor.visit_ident(id.as_mut())
     }
 
+    pub fn visit_iso_datetime<V: VisitMut + ?Sized>(visitor: &mut V, datetime: &mut IsoDateTime) {
+        visitor.visit_iso_date(datetime.as_mut());
+        visitor.visit_iso_time(datetime.as_mut());
+    }
+
     #[allow(unused_variables)]
-    pub fn visit_iso_date<V: VisitMut + ?Sized>(visitor: &mut V, date: &mut IsoDateTime) {}
+    pub fn visit_iso_date<V: VisitMut + ?Sized>(visitor: &mut V, date: &mut IsoDate) {}
+
+    #[allow(unused_variables)]
+    pub fn visit_iso_time<V: VisitMut + ?Sized>(visitor: &mut V, date: &mut IsoTime) {}
 
     pub fn visit_literal_property_value<V: VisitMut + ?Sized>(
         visitor: &mut V,
@@ -810,7 +858,7 @@ pub mod visit_mut {
             ReplacedBy(id) => visitor.visit_class_ident(id),
             Consider(id) => visitor.visit_class_ident(id),
             CreatedBy(s) => visitor.visit_unquoted_string(s),
-            CreationDate(dt) => visitor.visit_iso_date(dt),
+            CreationDate(dt) => visitor.visit_creation_date(dt),
         }
     }
 
@@ -865,7 +913,7 @@ pub mod visit_mut {
             ReplacedBy(id) => visitor.visit_relation_ident(id),
             Consider(id) => visitor.visit_ident(id),
             CreatedBy(s) => visitor.visit_unquoted_string(s),
-            CreationDate(dt) => visitor.visit_iso_date(dt),
+            CreationDate(dt) => visitor.visit_creation_date(dt),
             ExpandAssertionTo(s, xrefs) | ExpandExpressionTo(s, xrefs) => {
                 visitor.visit_quoted_string(s);
                 visitor.visit_xref_list(xrefs);
@@ -951,13 +999,12 @@ pub mod visit_mut {
 #[derive(Clone, Debug, Default)]
 pub struct IdCompactor {
     idspaces: HashMap<IdentPrefix, Url>,
+    interner: Cache,
 }
 
 impl IdCompactor {
     pub fn new() -> Self {
-        Self {
-            idspaces: HashMap::new(),
-        }
+        Self::default()
     }
 }
 
@@ -983,20 +1030,23 @@ impl VisitMut for IdCompactor {
             // find a prefix from the idspaces declared in the document
             for (prefix, url) in self.idspaces.iter() {
                 if u.as_str().starts_with(url.as_str()) {
-                    let local = &u.as_str()[url.as_str().len()..];
-                    new = Some(PrefixedIdent::new(prefix.as_str(), local));
+                    let local = self.interner.intern(&u.as_str()[url.as_str().len()..]);
+                    new = Some(PrefixedIdent::new(prefix.clone(), local));
                 }
             }
             // if none found, attempt to use the OBO factorisation
             const OBO_URL: &str = "http://purl.obolibrary.org/obo/";
-            if new.is_none() && u.as_str().starts_with(OBO_URL) {
-                let raw_id = &u.as_str()[OBO_URL.len()..];
-                if let Some(i) = raw_id.find('_') {
-                    // check we are not using a declared prefix (otherwise
-                    // the compaction/expansion would not roundtrip!)
-                    let prefix = IdentPrefix::new(&raw_id[..i]);
-                    if self.idspaces.get(&prefix).is_none() {
-                        new = Some(PrefixedIdent::new(prefix.as_str(), &raw_id[i + 1..]));
+            if new.is_none() {
+                if let Some(raw_id) = u.as_str().strip_prefix(OBO_URL) {
+                    if let Some(i) = raw_id.quickfind(b'_') {
+                        // check we are not using a declared prefix (otherwise
+                        // the compaction/expansion would not roundtrip!)
+                        // let prefix = IdentPrefix::new(self.intern(&raw_id[..i]));
+                        if self.idspaces.get(&raw_id[..i]).is_none() {
+                            let prefix = IdentPrefix::new(self.interner.intern(&raw_id[..i]));
+                            let local = self.interner.intern(&raw_id[i + 1..]);
+                            new = Some(PrefixedIdent::new(prefix, local));
+                        }
                     }
                 }
             }
@@ -1024,13 +1074,12 @@ impl VisitMut for IdCompactor {
 #[derive(Clone, Debug, Default)]
 pub struct IdDecompactor {
     idspaces: HashMap<IdentPrefix, Url>,
+    interner: Cache,
 }
 
 impl IdDecompactor {
     pub fn new() -> Self {
-        Self {
-            idspaces: HashMap::new(),
-        }
+        Self::default()
     }
 }
 
@@ -1052,21 +1101,19 @@ impl VisitMut for IdDecompactor {
         // the compacted id.
         let mut new: Option<Url> = None;
         if let Ident::Prefixed(p) = id {
-            if let Some(baseurl) = self.idspaces.get(p.prefix()) {
-                let newurl = format!("{}{}", baseurl, p.local());
-                new = Some(Url::from_str(&newurl).expect("invalid URL"));
-            } else {
-                let newurl = format!(
+            let new_url = match self.idspaces.get(p.prefix()) {
+                Some(base_url) => format!("{}{}", base_url.as_str(), p.local()),
+                None => format!(
                     "http://purl.obolibrary.org/obo/{}_{}",
                     p.prefix(),
                     p.local()
-                );
-                new = Some(Url::from_str(&newurl).expect("invalid URL"));
-            }
+                ),
+            };
+            new = Url::new(self.interner.intern(&new_url)).ok();
         }
 
-        if let Some(new_url) = new {
-            *id = Ident::Url(Box::new(new_url));
+        if let Some(new_id) = new {
+            *id = Ident::Url(Box::new(new_id));
         }
     }
 }
